@@ -1,4 +1,4 @@
-#!python
+#!/usr/bin/env python
 
 """
 Easy Install
@@ -28,6 +28,8 @@ import site
 import struct
 from glob import glob
 from distutils import log, dir_util
+from distutils.command.build_scripts import first_line_re
+
 
 import pkg_resources
 from setuptools import Command, _dont_write_bytecode
@@ -53,7 +55,7 @@ from pkg_resources import (
     VersionConflict, DEVELOP_DIST,
 )
 
-sys_executable = os.environ.get('__VENV_LAUNCHER__',
+sys_executable = os.environ.get('__PYVENV_LAUNCHER__',
     os.path.normpath(sys.executable))
 
 __all__ = [
@@ -644,6 +646,15 @@ Please make the appropriate changes for your system and try again.
     def process_distribution(self, requirement, dist, deps=True, *info):
         self.update_pth(dist)
         self.package_index.add(dist)
+        # First remove the dist from self.local_index, to avoid problems using
+        # old cached data in case its underlying file has been replaced.
+        #
+        # This is a quick-fix for a zipimporter caching issue in case the dist
+        # has been implemented as and already loaded from a zip file that got
+        # replaced later on. For more detailed information see setuptools issue
+        # #168 at 'http://bitbucket.org/pypa/setuptools/issue/168'.
+        if dist in self.local_index[dist.key]:
+            self.local_index.remove(dist)
         self.local_index.add(dist)
         self.install_egg_scripts(dist)
         self.installed_projects[dist.key] = dist
@@ -743,6 +754,8 @@ Please make the appropriate changes for your system and try again.
             return clean_template
 
         if is_script:
+            # See https://bitbucket.org/pypa/setuptools/issue/134 for info
+            #  on script file naming and downstream issues with SVR4
             template_name = 'script template.py'
             if dev_path:
                 template_name = template_name.replace('.py', ' (dev).py')
@@ -766,7 +779,7 @@ Please make the appropriate changes for your system and try again.
             f = open(target,"w"+mode)
             f.write(contents)
             f.close()
-            chmod(target, 0x1FF-mask) # 0777
+            chmod(target, 0o777-mask)
 
     def install_eggs(self, spec, dist_filename, tmpdir):
         # .egg dirs or files are already built, so just return them
@@ -1106,7 +1119,7 @@ See the setuptools documentation for the "develop" command for more info.
         self.byte_compile(to_compile)
         if not self.dry_run:
             for f in to_chmod:
-                mode = ((os.stat(f)[stat.ST_MODE]) | 0x16D) & 0xFED  # 0555, 07755
+                mode = ((os.stat(f)[stat.ST_MODE]) | 0o555) & 0o7755
                 chmod(f, mode)
 
     def byte_compile(self, to_compile):
@@ -1202,8 +1215,8 @@ Please make the appropriate changes for your system and try again."""
         home = convert_path(os.path.expanduser("~"))
         for name, path in iteritems(self.config_vars):
             if path.startswith(home) and not os.path.isdir(path):
-                self.debug_print("os.makedirs('%s', 0700)" % path)
-                os.makedirs(path, 0x1C0)    # 0700
+                self.debug_print("os.makedirs('%s', 0o700)" % path)
+                os.makedirs(path, 0o700)
 
     INSTALL_SCHEMES = dict(
         posix = dict(
@@ -1524,16 +1537,23 @@ class PthDistributions(Environment):
         else:
             return path
 
-def get_script_header(script_text, executable=sys_executable, wininst=False):
-    """Create a #! line, getting options (if any) from script_text"""
-    from distutils.command.build_scripts import first_line_re
+
+def _first_line_re():
+    """
+    Return a regular expression based on first_line_re suitable for matching
+    strings.
+    """
+    if isinstance(first_line_re.pattern, str):
+        return first_line_re
 
     # first_line_re in Python >=3.1.4 and >=3.2.1 is a bytes pattern.
-    if not isinstance(first_line_re.pattern, str):
-        first_line_re = re.compile(first_line_re.pattern.decode())
+    return re.compile(first_line_re.pattern.decode())
 
+
+def get_script_header(script_text, executable=sys_executable, wininst=False):
+    """Create a #! line, getting options (if any) from script_text"""
     first = (script_text+'\n').splitlines()[0]
-    match = first_line_re.match(first)
+    match = _first_line_re().match(first)
     options = ''
     if match:
         options = match.group(1) or ''
@@ -1563,20 +1583,34 @@ def auto_chmod(func, arg, exc):
     reraise(et, (ev[0], ev[1] + (" %s %s" % (func,arg))))
 
 def uncache_zipdir(path):
-    """Ensure that the importer caches dont have stale info for `path`"""
-    from zipimport import _zip_directory_cache as zdc
-    _uncache(path, zdc)
-    _uncache(path, sys.path_importer_cache)
+    """
+    Remove any globally cached zip file related data for `path`
 
-def _uncache(path, cache):
-    if path in cache:
-        del cache[path]
-    else:
-        path = normalize_path(path)
-        for p in cache:
-            if normalize_path(p)==path:
-                del cache[p]
-                return
+    Stale zipimport.zipimporter objects need to be removed when a zip file is
+    replaced as they contain cached zip file directory information. If they are
+    asked to get data from their zip file, they will use that cached
+    information to calculate the data location in the zip file. This calculated
+    location may be incorrect for the replaced zip file, which may in turn
+    cause the read operation to either fail or return incorrect data.
+
+    Note we have no way to clear any local caches from here. That is left up to
+    whomever is in charge of maintaining that cache.
+
+    """
+    normalized_path = normalize_path(path)
+    _uncache(normalized_path, zipimport._zip_directory_cache)
+    _uncache(normalized_path, sys.path_importer_cache)
+
+def _uncache(normalized_path, cache):
+    to_remove = []
+    prefix_len = len(normalized_path)
+    for p in cache:
+        np = normalize_path(p)
+        if (np.startswith(normalized_path) and
+                np[prefix_len:prefix_len + 1] in (os.sep, '')):
+            to_remove.append(p)
+    for p in to_remove:
+        del cache[p]
 
 def is_python(text, filename='<string>'):
     "Is this string a valid Python script?"
@@ -1862,7 +1896,7 @@ def rmtree(path, ignore_errors=False, onerror=auto_chmod):
         onerror(os.rmdir, path, sys.exc_info())
 
 def current_umask():
-    tmp = os.umask(0x12)    # 022
+    tmp = os.umask(0o022)
     os.umask(tmp)
     return tmp
 
